@@ -13,14 +13,14 @@
  *
  * You should have received a copy of the GNU Library General Public License
  * along with this library; see the file COPYING.LIB.  If not, write to
- * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
  */
 
 #ifndef __KWORD_LATEX_IMPORT_KWORDGENERATOR__
 #define __KWORD_LATEX_IMPORT_KWORDGENERATOR__
 
-#include <koStore.h>
+#include <KoStore.h>
 
 //#include "document.h"
 #include "element.h"
