@@ -1,12 +1,13 @@
 #ifndef _kchartWIZARDSELECTCHARTSUBTYPEPAGE_H
 #define _kchartWIZARDSELECTCHARTSUBTYPEPAGE_H
 
-#include <qradiobutton.h>
 
 #include "kchart_params.h"
 
 class QButtonGroup;
 class QPushButton;
+class QRadioButton;
+class QSpinBox;
 
 namespace KChart
 {
@@ -18,17 +19,21 @@ class KChartWizardSelectChartSubTypePage : public QWidget
   Q_OBJECT
 public:
   KChartWizardSelectChartSubTypePage( QWidget* parent, KChartPart* chart );
-  void changeSubTypeName( KDChartParams::ChartType _type);
+  void changeSubTypeName( KChartParams::ChartType _type);
+
   bool chartSubType;
 public slots:
   void apply();
 
 private:
-  KChartPart* _chart;
-  KDChartParams::ChartType _charttype;
-  QRadioButton* normal;
-  QRadioButton* stacked;
-  QRadioButton* percent;
+  KChartPart              *m_chart;
+  KChartParams::ChartType  m_charttype;
+
+  // Widgets
+  QRadioButton  *m_normal;
+  QRadioButton  *m_stacked;
+  QRadioButton  *m_percent;
+  QSpinBox      *m_numLines;
 };
 
 }  //KChart namespace
