@@ -14,8 +14,8 @@
 
    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
-   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-   Boston, MA 02111-1307, USA.
+   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
 */
 
 #include <qpainter.h>
@@ -40,6 +40,7 @@
 #include "formmanager.h"
 #include "commands.h"
 #include "events.h"
+#include "kexiflowlayout.h"
 
 using namespace KFormDesigner;
 
@@ -71,6 +72,10 @@ EventEater::eventFilter(QObject *, QEvent *ev)
 			//return true;
 		}
 	}
+//	else if(ev->type() == QEvent::ChildInserted) {
+		// widget's children have changed, we need to reinstall filter
+//		installRecursiveEventFilter(m_widget, this);
+//	}
 
 	return m_container->eventFilter(m_widget, ev);
 }
@@ -85,6 +90,9 @@ EventEater::~EventEater()
 
 Container::Container(Container *toplevel, QWidget *container, QObject *parent, const char *name)
 : QObject(parent, name)
+, m_insertBegin(-1,-1)
+, m_mousePressEventReceived(false)
+, m_mouseReleaseEvent(QEvent::None, QPoint(), 0, 0)
 {
 	m_container = container;
 	m_toplevel = toplevel;
@@ -97,7 +105,8 @@ Container::Container(Container *toplevel, QWidget *container, QObject *parent, c
 	m_state = DoingNothing;
 
 	QCString classname = container->className();
-	if((classname == "HBox") || (classname == "Grid") || (classname == "VBox"))
+	if((classname == "HBox") || (classname == "Grid") || (classname == "VBox") ||
+		(classname == "HFlow")  || (classname == "VFlow"))
 		m_margin = 4; // those containers don't have frames, so little margin
 	else
 		m_margin = m_form ? m_form->defaultMargin() : 0;
@@ -105,7 +114,7 @@ Container::Container(Container *toplevel, QWidget *container, QObject *parent, c
 
 	if(toplevel)
 	{
-		ObjectTreeItem *it = new ObjectTreeItem(m_form->manager()->lib()->displayName(classname),
+		ObjectTreeItem *it = new ObjectTreeItem(m_form->library()->displayName(classname),
 			widget()->name(), widget(), this, this);
 		setObjectTree(it);
 
@@ -140,25 +149,30 @@ Container::setForm(Form *form)
 bool
 Container::eventFilter(QObject *s, QEvent *e)
 {
+//	kdDebug() << e->type() << endl;
 	switch(e->type())
 	{
 		case QEvent::MouseButtonPress:
 		{
+			m_insertBegin = QPoint(-1, -1);
+			m_mousePressEventReceived = true;
+
 			kdDebug() << "QEvent::MouseButtonPress sender object = " << s->name()
 				<< "of type " << s->className() << endl;
-			kdDebug() << "QEvent::MouseButtonPress this          = " << this->name() << endl;
+			kdDebug() << "QEvent::MouseButtonPress this          = " << name() << endl;
 
 			m_moving = static_cast<QWidget*>(s);
 			QMouseEvent *mev = static_cast<QMouseEvent*>(e);
 			m_grab = QPoint(mev->x(), mev->y());
 
 			// we are drawing a connection
-			if(m_form->manager()->isCreatingConnection())  {
+			if(FormManager::self()->isCreatingConnection())  {
 				drawConnection(mev);
 				return true;
 			}
 
-			if(((mev->state() == ControlButton) || (mev->state() == ShiftButton)) && (!m_form->manager()->isInserting())) // multiple selection mode
+			if(((mev->state() == ControlButton) || (mev->state() == ShiftButton)) 
+				&& (!FormManager::self()->isInserting())) // multiple selection mode
 			{
 				if(m_form->selectedWidgets()->findRef(m_moving) != -1) // widget is already selected
 				{
@@ -185,18 +199,18 @@ Container::eventFilter(QObject *s, QEvent *e)
 				setSelectedWidget(m_moving, false, (mev->button() == RightButton));
 
 			// we are inserting a widget or drawing a selection rect in the form
-			if((/*s == m_container &&*/ m_form->manager()->isInserting()) || ((s == m_container) && !m_toplevel))
+			if((/*s == m_container &&*/ FormManager::self()->isInserting()) || ((s == m_container) && !m_toplevel))
 			{
 				int tmpx,tmpy;
-				if(!m_form->manager()->snapWidgetsToGrid() || (mev->state() == (LeftButton|ControlButton|AltButton)))
+				if(!FormManager::self()->snapWidgetsToGrid() || (mev->state() == (LeftButton|ControlButton|AltButton)))
 				{
 					tmpx = mev->x();
 					tmpy = mev->y();
 				}
 				else
 				{
-					int gridX = m_form->gridX();
-					int gridY = m_form->gridY();
+					int gridX = m_form->gridSize();
+					int gridY = m_form->gridSize();
 					tmpx = int( (float)mev->x() / ((float)gridX) + 0.5 ); // snap to grid
 					tmpx *= gridX;
 					tmpy = int( (float)mev->y() / ((float)gridY) + 0.5 );
@@ -207,79 +221,39 @@ Container::eventFilter(QObject *s, QEvent *e)
 				if(m_form->formWidget())
 					m_form->formWidget()->initBuffer();
 
-				if(!m_form->manager()->isInserting())
+				if(!FormManager::self()->isInserting())
 					m_state = DrawingSelectionRect;
-				return true;
+			}
+			else {
+				if(s->inherits("QTabWidget")) // to allow changing page by clicking tab
+					return false;
 			}
 
-			if(s->inherits("QTabWidget")) // to allow changing page by clicking tab
-				return false;
+			if (m_objectForMouseReleaseEvent) {
+				const bool res = handleMouseReleaseEvent(m_objectForMouseReleaseEvent, &m_mouseReleaseEvent);
+				m_objectForMouseReleaseEvent = 0;
+				return res;
+			}
 			return true;
 		}
 
 		case QEvent::MouseButtonRelease:
 		{
 			QMouseEvent *mev = static_cast<QMouseEvent*>(e);
-			if(m_form->manager()->isInserting()) // we insert the widget at cursor pos
-			{
-				if(m_form->formWidget())
-					m_form->formWidget()->clearForm();
-				KCommand *com = new InsertWidgetCommand(this/*, mev->pos()*/);
-				m_form->addCommand(com, true);
-				m_insertRect = QRect();
+			if (!m_mousePressEventReceived) {
+				m_mouseReleaseEvent = *mev;
+				m_objectForMouseReleaseEvent = s;
 				return true;
 			}
-			else if(s == m_container && !m_toplevel && (mev->button() != RightButton) && m_insertRect.isValid()) // we are drawing a rect to select widgets
-			{
-				drawSelectionRect(mev);
-				return true;
-			}
-			if(mev->button() == RightButton) // Right-click -> context menu
-			{
-				m_form->manager()->createContextMenu(static_cast<QWidget*>(s), this);
-			}
-			else if(mev->state() == (Qt::LeftButton|Qt::ControlButton))// && (m_copyRect.isValid()))
-			{
-				// copying a widget by Ctrl+dragging
-
-				if(m_form->formWidget())
-					m_form->formWidget()->clearForm();
-				if(s == m_container) // should have no effect on form
-					return true;
-
-				// prevent accidental copying of widget (when moving mouse a little while selecting)
-				if( ( (mev->pos().x() - m_grab.x()) < form()->gridX() &&  (m_grab.x() - mev->pos().x()) < form()->gridX() ) &&
-					( (mev->pos().y() - m_grab.y()) < form()->gridY() &&  (m_grab.y() - mev->pos().y()) < form()->gridY() ) )
-				{
-					kdDebug() << "The widget has not been moved. No copying" << endl;
-					return true;
-				}
-
-				m_form->setInteractiveMode(false);
-				// We simulate copy and paste
-				m_form->manager()->copyWidget();
-				if(m_form->selectedWidgets()->count() > 1)
-					m_form->manager()->setInsertPoint( mev->pos() );
-				else
-					m_form->manager()->setInsertPoint( static_cast<QWidget*>(s)->mapTo(m_container, mev->pos() - m_grab) );
-				m_form->manager()->pasteWidget();
-				m_form->setInteractiveMode(true);
-
-				//m_initialPos = QPoint();
-			}
-			else if(m_state == MovingWidget) // one widget has been moved, so we need to update the layout
-				reloadLayout();
-
-			// cancel copying as user released Ctrl before releasing mouse button
-			m_insertRect = QRect();
-			m_state = DoingNothing;
-			return true; // eat
+			m_mousePressEventReceived = false;
+			m_objectForMouseReleaseEvent = 0;
+			return handleMouseReleaseEvent(s, mev);
 		}
 
 		case QEvent::MouseMove:
 		{
 			QMouseEvent *mev = static_cast<QMouseEvent*>(e);
-			if(m_form->manager()->isInserting() && ((mev->state() == LeftButton) || (mev->state() == (LeftButton|ControlButton)) ||
+			if(m_insertBegin!=QPoint(-1,-1) && FormManager::self()->isInserting() && ((mev->state() == LeftButton) || (mev->state() == (LeftButton|ControlButton)) ||
 			(mev->state() == (LeftButton|ControlButton|AltButton)) || (mev->state() == (LeftButton|ShiftButton)) ) )
 			// draw the insert rect
 			{
@@ -287,16 +261,16 @@ Container::eventFilter(QObject *s, QEvent *e)
 				return true;
 			}
 			// Creating a connection, we highlight sender and receiver, and we draw a link between them
-			else if(m_form->manager()->isCreatingConnection() && !m_form->manager()->createdConnection()->sender().isNull())
+			else if(FormManager::self()->isCreatingConnection() && !FormManager::self()->createdConnection()->sender().isNull())
 			{
-				ObjectTreeItem *tree = m_form->objectTree()->lookup(m_form->manager()->createdConnection()->sender());
+				ObjectTreeItem *tree = m_form->objectTree()->lookup(FormManager::self()->createdConnection()->sender());
 				if(!tree || !tree->widget())
 					return true;
 
 				if(m_form->formWidget() && (tree->widget() != s))
 					m_form->formWidget()->highlightWidgets(tree->widget(), static_cast<QWidget*>(s));
 			}
-			else if(s == m_container && !m_toplevel && (mev->state() != ControlButton) && !m_form->manager()->isCreatingConnection()) // draw the selection rect
+			else if(m_insertBegin!=QPoint(-1,-1) && s == m_container && !m_toplevel && (mev->state() != ControlButton) && !FormManager::self()->isCreatingConnection()) // draw the selection rect
 			{
 				if((mev->state() != LeftButton) || /*m_inlineEditing*/ m_state == InlineEditing)
 					return true;
@@ -321,7 +295,7 @@ Container::eventFilter(QObject *s, QEvent *e)
 				return true;
 			}
 			else if( ( (mev->state() == Qt::LeftButton) || (mev->state() == (LeftButton|ControlButton|AltButton)) )
-			  && !m_form->manager()->isInserting() && (m_state != CopyingWidget)) // we are dragging the widget(s) to move it
+			  && !FormManager::self()->isInserting() && (m_state != CopyingWidget)) // we are dragging the widget(s) to move it
 			{
 				if(!m_toplevel && m_moving == m_container) // no effect for form
 					return false;
@@ -339,8 +313,8 @@ Container::eventFilter(QObject *s, QEvent *e)
 		{
 			if(s != m_container)
 				return false;
-			int gridX = m_form->gridX();
-			int gridY = m_form->gridY();
+			int gridX = m_form->gridSize();
+			int gridY = m_form->gridSize();
 
 			QPainter p(m_container);
 			p.setPen(QPen(white, 2));
@@ -366,6 +340,7 @@ Container::eventFilter(QObject *s, QEvent *e)
 			break;
 		}
 
+		//case QEvent::AccelOverride:
 		case QEvent::KeyPress:
 		{
 			QKeyEvent *kev = static_cast<QKeyEvent*>(e);
@@ -382,14 +357,14 @@ Container::eventFilter(QObject *s, QEvent *e)
 					w = m_moving;
 				else
 					w = m_form->selectedWidgets()->last();
-				m_form->manager()->lib()->startEditing(w->className(), w, this);
+				m_form->library()->startEditing(w->className(), w, this);
 			}
 			else if(kev->key() == Key_Escape)
 			{
-				if(m_form->manager()->isCreatingConnection())
-					m_form->manager()->stopCreatingConnection();
-				else if(m_form->manager()->isInserting())
-					m_form->manager()->stopInsert();
+				if(FormManager::self()->isCreatingConnection())
+					FormManager::self()->stopCreatingConnection();
+				else if(FormManager::self()->isInserting())
+					FormManager::self()->stopInsert();
 				return true;
 			}
 			else if((kev->key() == Key_Control) && (m_state == MovingWidget))
@@ -402,38 +377,40 @@ Container::eventFilter(QObject *s, QEvent *e)
 				eventFilter(m_moving, mev);
 				delete mev;
 			}
-			else if(kev->key() == m_form->manager()->contextMenuKey())
+			else if(kev->key() == FormManager::self()->contextMenuKey())
 			{
-					m_form->manager()->createContextMenu(static_cast<QWidget*>(s), this, false);
+					FormManager::self()->createContextMenu(static_cast<QWidget*>(s), this, false);
 					return true;
 			}
 			else if (kev->key() == Key_Delete)
 			{
-				m_form->manager()->deleteWidget();
+				FormManager::self()->deleteWidget();
 				return true;
 			}
 			// directional buttons move the widget
 			else if(kev->key() == Key_Left){ // move the widget of gridX to the left
-				moveSelectedWidgetsBy(-form()->gridX(), 0);
+				moveSelectedWidgetsBy(-form()->gridSize(), 0);
 				return true;
 			}
 			else if(kev->key() == Key_Right){ // move the widget of gridX to the right
-				moveSelectedWidgetsBy(form()->gridX(), 0);
+				moveSelectedWidgetsBy(form()->gridSize(), 0);
 				return true;
 			}
 			else if(kev->key() == Key_Up){ // move the widget of gridY to the top
-				moveSelectedWidgetsBy(0, - form()->gridY());
+				moveSelectedWidgetsBy(0, - form()->gridSize());
 				return true;
 			}
 			else if(kev->key() == Key_Down){ // move the widget of gridX to the bottom
-				moveSelectedWidgetsBy(0, form()->gridY());
+				moveSelectedWidgetsBy(0, form()->gridSize());
 				return true;
 			}
 			else if((kev->key() == Key_Tab) || (kev->key() == Key_BackTab)){
 				ObjectTreeItem *item = form()->objectTree()->lookup(form()->selectedWidgets()->first()->name());
-				if(!item) return true;
+				if(!item || !item->parent())
+					return true;
 				ObjectTreeList *list = item->parent()->children();
-				if(list->count() == 1) return true;
+				if(list->count() == 1)
+					return true;
 				int index = list->findRef(item);
 
 				if(kev->key() == Key_BackTab){
@@ -443,7 +420,7 @@ Container::eventFilter(QObject *s, QEvent *e)
 						index = index - 1;
 				}
 				else  {
-					if(index == list->count() - 1) // go back to the first item
+					if(index == int(list->count() - 1)) // go back to the first item
 						index = 0;
 					else
 						index = index + 1;
@@ -477,7 +454,7 @@ Container::eventFilter(QObject *s, QEvent *e)
 
 			//m_inlineEditing = true;
 			m_state = InlineEditing;
-			m_form->manager()->lib()->startEditing(w->className(), w, this);
+			m_form->library()->startEditing(w->className(), w, this);
 			return true;
 		}
 
@@ -486,9 +463,9 @@ Container::eventFilter(QObject *s, QEvent *e)
 		case QEvent::Leave:
 		case QEvent::FocusIn:
 		case QEvent::FocusOut:
-		case QEvent::DragEnter:
-		case QEvent::DragMove:
-		case QEvent::DragLeave:
+//		case QEvent::DragEnter:
+//		case QEvent::DragMove:
+//		case QEvent::DragLeave:
 			return true; // eat them
 
 		default:
@@ -497,8 +474,69 @@ Container::eventFilter(QObject *s, QEvent *e)
 	return false;
 }
 
+bool
+Container::handleMouseReleaseEvent(QObject *s, QMouseEvent *mev)
+{
+	if(FormManager::self()->isInserting()) // we insert the widget at cursor pos
+	{
+		if(m_form->formWidget())
+			m_form->formWidget()->clearForm();
+		KCommand *com = new InsertWidgetCommand(this/*, mev->pos()*/);
+		m_form->addCommand(com, true);
+		m_insertBegin = QPoint(-1,-1);
+		m_insertRect = QRect();
+		return true;
+	}
+	else if(s == m_container && !m_toplevel && (mev->button() != RightButton) && m_insertRect.isValid()) // we are drawing a rect to select widgets
+	{
+		drawSelectionRect(mev);
+		return true;
+	}
+	if(mev->button() == RightButton) // Right-click -> context menu
+	{
+		FormManager::self()->createContextMenu(static_cast<QWidget*>(s), this);
+	}
+	else if(mev->state() == (Qt::LeftButton|Qt::ControlButton))// && (m_copyRect.isValid()))
+	{
+		// copying a widget by Ctrl+dragging
+
+		if(m_form->formWidget())
+			m_form->formWidget()->clearForm();
+		if(s == m_container) // should have no effect on form
+			return true;
+
+		// prevent accidental copying of widget (when moving mouse a little while selecting)
+		if( ( (mev->pos().x() - m_grab.x()) < form()->gridSize() &&  (m_grab.x() - mev->pos().x()) < form()->gridSize() ) &&
+			( (mev->pos().y() - m_grab.y()) < form()->gridSize() &&  (m_grab.y() - mev->pos().y()) < form()->gridSize() ) )
+		{
+			kdDebug() << "The widget has not been moved. No copying" << endl;
+			return true;
+		}
+
+		m_form->setInteractiveMode(false);
+		// We simulate copy and paste
+		FormManager::self()->copyWidget();
+		if(m_form->selectedWidgets()->count() > 1)
+			FormManager::self()->setInsertPoint( mev->pos() );
+		else
+			FormManager::self()->setInsertPoint( static_cast<QWidget*>(s)->mapTo(m_container, mev->pos() - m_grab) );
+		FormManager::self()->pasteWidget();
+		m_form->setInteractiveMode(true);
+
+		//m_initialPos = QPoint();
+	}
+	else if(m_state == MovingWidget) // one widget has been moved, so we need to update the layout
+		reloadLayout();
+
+	// cancel copying as user released Ctrl before releasing mouse button
+	m_insertBegin = QPoint(-1,-1);
+	m_insertRect = QRect();
+	m_state = DoingNothing;
+	return true; // eat
+}
+
 void
-Container::setSelectedWidget(QWidget *w, bool add, bool dontRaise)
+Container::setSelectedWidget(QWidget *w, bool add, bool dontRaise, bool moreWillBeSelected)
 {
 	if (w)
 		kdDebug() << "slotSelectionChanged " << w->name()<< endl;
@@ -509,7 +547,7 @@ Container::setSelectedWidget(QWidget *w, bool add, bool dontRaise)
 		return;
 	}
 
-	m_form->setSelectedWidget(w, add, dontRaise);
+	m_form->setSelectedWidget(w, add, dontRaise, moreWillBeSelected);
 }
 
 void
@@ -535,10 +573,9 @@ Container::deleteWidget(QWidget *w)
 {
 	if(!w)
 		return;
-
-	kdDebug() << "Deleting a widget: " << w->name() << endl;
+//	kdDebug() << "Deleting a widget: " << w->name() << endl;
 	m_form->objectTree()->removeItem(w->name());
-	m_form->manager()->deleteWidgetLater( w );
+	FormManager::self()->deleteWidgetLater( w );
 	m_form->setSelectedWidget(m_container);
 }
 
@@ -563,10 +600,6 @@ Container::setLayout(LayoutType type)
 
 	switch(type)
 	{
-		case NoLayout:
-		{
-			return;
-		}
 		case HBox:
 		{
 			m_layout = (QLayout*) new QHBoxLayout(m_container, m_margin, m_spacing);
@@ -584,12 +617,30 @@ Container::setLayout(LayoutType type)
 			createGridLayout();
 			break;
 		}
+		case  HFlow:
+		{
+			KexiFlowLayout *flow = new KexiFlowLayout(m_container,m_margin, m_spacing);
+			flow->setOrientation(Horizontal);
+			m_layout = (QLayout*)flow;
+			createFlowLayout();
+			break;
+		}
+		case VFlow:
+		{
+			KexiFlowLayout *flow = new KexiFlowLayout(m_container,m_margin, m_spacing);
+			flow->setOrientation(Vertical);
+			m_layout = (QLayout*)flow;
+			createFlowLayout();
+			break;
+		}
 		default:
 		{
-			kdDebug() << "WRONG LAYOUT TYPE " << endl;
+			m_layType = NoLayout;
 			return;
 		}
 	}
+	m_container->setGeometry(m_container->geometry()); // just update layout
+	m_layout->activate();
 }
 
 void
@@ -612,12 +663,74 @@ Container::createBoxLayout(WidgetList *list)
 	for(QWidget *obj = list->first(); obj; obj = list->next())
 		layout->addWidget(obj);
 	delete list;
-
-	layout->activate();
 }
 
 void
-Container::createGridLayout()
+Container::createFlowLayout()
+{
+	KexiFlowLayout *flow = dynamic_cast<KexiFlowLayout*>(m_layout);
+	if(!flow || m_tree->children()->isEmpty())
+		return;
+
+	const int offset = 15;
+	WidgetList *list=0, *list2=0;
+	if(flow->orientation() == Horizontal) {
+		list = new VerWidgetList();
+		list2 = new HorWidgetList();
+	}
+	else {
+		list = new HorWidgetList();
+		list2 = new VerWidgetList();
+	}
+
+	// fill the list
+	for(ObjectTreeItem *tree = m_tree->children()->first(); tree; tree = m_tree->children()->next())
+		list->append( tree->widget());
+	list->sort();
+
+	if(flow->orientation() == Horizontal) {
+		int y = list->first()->y();
+		for(QWidget *w = list->first(); w; w = list->next()) {
+			if( (w->y() > y +offset)) {
+				// start a new line
+				list2->sort();
+				for(QWidget *obj = list2->first(); obj; obj = list2->next())
+					flow->add(obj);
+				list2->clear();
+				y = w->y();
+			}
+			list2->append(w);
+		}
+
+		list2->sort(); // don't forget the last line
+		for(QWidget *obj = list2->first(); obj; obj = list2->next())
+			flow->add(obj);
+	}
+	else {
+		int x = list->first()->x();
+		for(QWidget *w = list->first(); w; w = list->next()) {
+			if( (w->x() > x +offset)) {
+				// start a new column
+				list2->sort();
+				for(QWidget *obj = list2->first(); obj; obj = list2->next())
+					flow->add(obj);
+				list2->clear();
+				x = w->x();
+			}
+			list2->append(w);
+		}
+
+		list2->sort(); // don't forget the last column
+		for(QWidget *obj = list2->first(); obj; obj = list2->next())
+			flow->add(obj);
+	}
+
+	delete list;
+	delete list2;
+}
+
+void
+Container::createGridLayout(bool testOnly)
 {
 	//Those lists sort widgets by y and x
 	VerWidgetList *vlist = new VerWidgetList();
@@ -638,26 +751,28 @@ Container::createGridLayout()
 
 	// First we need to make sure that two widgets won't be in the same row,
 	// ie that no widget overlap another one
-	for(WidgetListIterator it(*vlist); it.current() != 0; ++it)
-	{
-		QWidget *w = it.current();
-		WidgetListIterator it2 = it;
+	if(!testOnly) {
+		for(WidgetListIterator it(*vlist); it.current() != 0; ++it)
+		{
+			QWidget *w = it.current();
+			WidgetListIterator it2 = it;
 
-		for(; it2.current() != 0; ++it2) {
-			QWidget *nextw = it2.current();
-			if((w->y() >= nextw->y()) || (nextw->y() >= w->geometry().bottom()))
-				break;
+			for(; it2.current() != 0; ++it2) {
+				QWidget *nextw = it2.current();
+				if((w->y() >= nextw->y()) || (nextw->y() >= w->geometry().bottom()))
+					break;
 
-			if(!w->geometry().intersects(nextw->geometry()))
-				break;
-			// If the geometries of the two widgets intersect each other,
-			// we move one of the widget to the rght or bottom of the other
-			if((nextw->y() - w->y()) > abs(nextw->x() - w->x()))
-				nextw->move(nextw->x(), w->geometry().bottom()+1);
-			else if(nextw->x() >= w->x())
-				nextw->move(w->geometry().right()+1, nextw->y());
-			else
-				w->move(nextw->geometry().right()+1, nextw->y());
+				if(!w->geometry().intersects(nextw->geometry()))
+					break;
+				// If the geometries of the two widgets intersect each other,
+				// we move one of the widget to the rght or bottom of the other
+				if((nextw->y() - w->y()) > abs(nextw->x() - w->x()))
+					nextw->move(nextw->x(), w->geometry().bottom()+1);
+				else if(nextw->x() >= w->x())
+					nextw->move(w->geometry().right()+1, nextw->y());
+				else
+					w->move(nextw->geometry().right()+1, nextw->y());
+			}
 		}
 	}
 
@@ -716,8 +831,11 @@ Container::createGridLayout()
 	kdDebug() << "the new grid will have n columns: n == " << cols.size() << endl;
 
 	// We create the layout ..
-	QGridLayout *layout = new QGridLayout(m_container, rows.size(), cols.size(), m_margin, m_spacing, "grid");
-	m_layout = (QLayout*)layout;
+	QGridLayout *layout=0;
+	if(!testOnly) {
+		layout = new QGridLayout(m_container, rows.size(), cols.size(), m_margin, m_spacing, "grid");
+		m_layout = (QLayout*)layout;
+	}
 
 	// .. and we fill it with widgets
 	for(WidgetListIterator it(*vlist); it.current() != 0; ++it)
@@ -781,20 +899,19 @@ Container::createGridLayout()
 		 // " and will go to the col " << endcol << endl;
 
 		ObjectTreeItem *item = m_form->objectTree()->lookup(w->name());
-		if(!endrow && !endcol)
-		{
+		if(!endrow && !endcol) {
+			if(!testOnly)
+				layout->addWidget(w, wrow, wcol);
 			item->setGridPos(wrow, wcol, 0, 0);
-			layout->addWidget(w, wrow, wcol);
 		}
-		else
-		{
+		else {
 			if(!endcol)  endcol = wcol;
 			if(!endrow)  endrow = wrow;
-			layout->addMultiCellWidget(w, wrow, endrow, wcol, endcol);
+			if(!testOnly)
+				layout->addMultiCellWidget(w, wrow, endrow, wcol, endcol);
 			item->setGridPos(wrow, wcol, endrow-wrow+1, endcol-wcol+1);
 		}
 	}
-	layout->activate();
 }
 
 QString
@@ -805,6 +922,8 @@ Container::layoutTypeToString(int type)
 		case HBox: return "HBox";
 		case VBox: return "VBox";
 		case Grid: return "Grid";
+		case HFlow: return "HFlow";
+		case VFlow: return "VFlow";
 		default:   return "NoLayout";
 	}
 }
@@ -812,9 +931,11 @@ Container::layoutTypeToString(int type)
 Container::LayoutType
 Container::stringToLayoutType(const QString &name)
 {
-	if(name == "HBox")        return HBox;
-	if(name == "VBox")        return VBox;
-	if(name == "Grid")        return Grid;
+	if(name == "HBox") return HBox;
+	if(name == "VBox") return VBox;
+	if(name == "Grid") return Grid;
+	if(name == "HFlow")  return HFlow;
+	if(name == "VFlow")  return VFlow;
 	return NoLayout;
 }
 
@@ -824,39 +945,39 @@ Container::drawConnection(QMouseEvent *mev)
 {
 	if(mev->button() != LeftButton)
 	{
-		m_form->manager()->resetCreatedConnection();
+		FormManager::self()->resetCreatedConnection();
 		return;
 	}
 	// First click, we select the sender and display menu to choose signal
-	if(m_form->manager()->createdConnection()->sender().isNull())
+	if(FormManager::self()->createdConnection()->sender().isNull())
 	{
-		m_form->manager()->createdConnection()->setSender(m_moving->name());
+		FormManager::self()->createdConnection()->setSender(m_moving->name());
 		if(m_form->formWidget())
 		{
 			m_form->formWidget()->initBuffer();
 			m_form->formWidget()->highlightWidgets(m_moving, 0/*, QPoint()*/);
 		}
-		m_form->manager()->createSignalMenu(m_moving);
+		FormManager::self()->createSignalMenu(m_moving);
 		return;
 	}
 	// the user clicked outside the menu, we cancel the connection
-	if(m_form->manager()->createdConnection()->signal().isNull())
+	if(FormManager::self()->createdConnection()->signal().isNull())
 	{
-		m_form->manager()->stopCreatingConnection();
+		FormManager::self()->stopCreatingConnection();
 		return;
 	}
 	// second click to choose the receiver
-	if(m_form->manager()->createdConnection()->receiver().isNull())
+	if(FormManager::self()->createdConnection()->receiver().isNull())
 	{
-		m_form->manager()->createdConnection()->setReceiver(m_moving->name());
-		m_form->manager()->createSlotMenu(m_moving);
+		FormManager::self()->createdConnection()->setReceiver(m_moving->name());
+		FormManager::self()->createSlotMenu(m_moving);
 		m_container->repaint();
 		return;
 	}
 	// the user clicked outside the menu, we cancel the connection
-	if(m_form->manager()->createdConnection()->slot().isNull())
+	if(FormManager::self()->createdConnection()->slot().isNull())
 	{
-		m_form->manager()->stopCreatingConnection();
+		FormManager::self()->stopCreatingConnection();
 		return;
 	}
 }
@@ -873,16 +994,22 @@ Container::drawSelectionRect(QMouseEvent *mev)
 	int boty = (m_insertBegin.y() > mev->y()) ? m_insertBegin.y() : mev->y();
 	QRect r = QRect(QPoint(topx, topy), QPoint(botx, boty));
 
-	QWidget *w=0;
 	setSelectedWidget(m_container, false);
+	QWidget *widgetToSelect = 0;
 	// We check which widgets are in the rect and select them
 	for(ObjectTreeItem *item = m_tree->children()->first(); item; item = m_tree->children()->next())
 	{
-		w = item->widget();
-		if(!w) continue;
-		if(w->geometry().intersects(r) && w != m_container)
-			setSelectedWidget(w, true);
+		QWidget *w = item->widget();
+		if(!w)
+			continue;
+		if(w->geometry().intersects(r) && w != m_container) {
+			if (widgetToSelect)
+				setSelectedWidget(widgetToSelect, true/*add*/, false/*raise*/, true/*moreWillBeSelected*/);
+			widgetToSelect = w; //select later
+		}
 	}
+	if (widgetToSelect) //the last one left
+		setSelectedWidget(widgetToSelect, true/*add*/, false/*raise*/, false/*!moreWillBeSelected*/);
 
 	m_insertRect = QRect();
 	m_state = DoingNothing;
@@ -894,9 +1021,9 @@ Container::drawInsertRect(QMouseEvent *mev, QObject *s)
 {
 	int tmpx, tmpy;
 	QPoint pos = static_cast<QWidget*>(s)->mapTo(m_container, mev->pos());
-	int gridX = m_form->gridX();
-	int gridY = m_form->gridY();
-	if(!m_form->manager()->snapWidgetsToGrid() || (mev->state() == (LeftButton|ControlButton|AltButton)) )
+	int gridX = m_form->gridSize();
+	int gridY = m_form->gridSize();
+	if(!FormManager::self()->snapWidgetsToGrid() || (mev->state() == (LeftButton|ControlButton|AltButton)) )
 	{
 		tmpx = pos.x();
 		tmpy = pos.y();
@@ -916,15 +1043,15 @@ Container::drawInsertRect(QMouseEvent *mev, QObject *s)
 	m_insertRect = QRect(QPoint(topx, topy), QPoint(botx, boty));
 
 	if(m_insertRect.x() < 0)
-		m_insertRect.moveLeft(0);
+		m_insertRect.setLeft(0);
 	if(m_insertRect.y() < 0)
-		m_insertRect.moveTop(0);
+		m_insertRect.setTop(0);
 	if(m_insertRect.right() > m_container->width())
-		m_insertRect.moveRight(m_container->width());
+		m_insertRect.setRight(m_container->width());
 	if(m_insertRect.bottom() > m_container->height())
-		m_insertRect.moveBottom(m_container->height());
+		m_insertRect.setBottom(m_container->height());
 
-	if(m_form->manager()->isInserting() && m_insertRect.isValid())
+	if(FormManager::self()->isInserting() && m_insertRect.isValid())
 	{
 		if(m_form->formWidget())
 		{
@@ -940,7 +1067,7 @@ Container::drawCopiedWidgetRect(QMouseEvent *mev)
 {
 	// We've been dragging a widget, but Ctrl was hold, so we start copy
 	if(m_state == MovingWidget)  {
-		//m_form->manager()->undo(); // undo last moving
+		//FormManager::self()->undo(); // undo last moving
 		//m_moving->move(m_initialPos);
 		if(m_form->formWidget())  {
 			m_container->repaint();
@@ -971,8 +1098,11 @@ Container::drawCopiedWidgetRect(QMouseEvent *mev)
 void
 Container::moveSelectedWidgetsBy(int realdx, int realdy, QMouseEvent *mev)
 {
-	int gridX = m_form->gridX();
-	int gridY = m_form->gridY();
+	if (m_form->selectedWidget() == m_form->widget())
+		return; //do not move top-level widget
+
+	const int gridX = m_form->gridSize();
+	const int gridY = m_form->gridSize();
 	int dx=realdx, dy=realdy;
 
 	for(QWidget *w = m_form->selectedWidgets()->first(); w; w = m_form->selectedWidgets()->next())
@@ -1014,7 +1144,7 @@ Container::moveSelectedWidgetsBy(int realdx, int realdy, QMouseEvent *mev)
 		}
 
 		int tmpx, tmpy;
-		if(!m_form->manager()->snapWidgetsToGrid() || (mev && mev->state() == (LeftButton|ControlButton|AltButton)) )
+		if(!FormManager::self()->snapWidgetsToGrid() || (mev && mev->state() == (LeftButton|ControlButton|AltButton)) )
 		{
 			tmpx = w->x() + dx;
 			tmpy = w->y() + dy;
@@ -1030,4 +1160,23 @@ Container::moveSelectedWidgetsBy(int realdx, int realdy, QMouseEvent *mev)
 	}
 }
 
+////////////
+
+DesignTimeDynamicChildWidgetHandler::DesignTimeDynamicChildWidgetHandler()
+ : m_item(0)
+{
+}
+
+DesignTimeDynamicChildWidgetHandler::~DesignTimeDynamicChildWidgetHandler()
+{
+}
+
+void
+DesignTimeDynamicChildWidgetHandler::childWidgetAdded(QWidget* w)
+{
+	if (m_item) {
+		installRecursiveEventFilter(w, m_item->eventEater());
+	}
+}
+
 #include "container.moc"
