@@ -42,16 +42,8 @@
 #include <kdebug.h>
 #include <ksavefile.h>
 
-//-------------------------------------------------------------
 
-KexiCSVExportWizard::Options::Options()
- : mode(File), itemId(0)
-{
-}
-
-//-------------------------------------------------------------
-
-KexiCSVExportWizard::KexiCSVExportWizard( const Options& options,
+KexiCSVExportWizard::KexiCSVExportWizard( const KexiCSVExport::Options& options,
 	KexiMainWindow* mainWin, QWidget * parent, const char * name )
  : KWizard(parent, name)
  , m_options(options)
@@ -60,10 +52,11 @@ KexiCSVExportWizard::KexiCSVExportWizard( const Options& options,
  , m_mainWin(mainWin)
  , m_fileSavePage(0)
  , m_defaultsBtn(0)
+ , m_rowCount(-1)
  , m_rowCountDetermined(false)
  , m_cancelled(false)
 {
-	if (m_options.mode==Clipboard) {
+	if (m_options.mode==KexiCSVExport::Clipboard) {
 		finishButton()->setText(i18n("Copy"));
 		backButton()->hide();
 	}
@@ -77,7 +70,7 @@ KexiCSVExportWizard::KexiCSVExportWizard( const Options& options,
 	m_tableOrQuery = new KexiDB::TableOrQuerySchema(
 		m_mainWin->project()->dbConnection(), m_options.itemId);
 	if (m_tableOrQuery->table()) {
-		if (m_options.mode==Clipboard) {
+		if (m_options.mode==KexiCSVExport::Clipboard) {
 			setCaption(i18n("Copy Data From Table to Clipboard"));
 			infoLblFromText = i18n("Copying data from table:");
 		}
@@ -87,7 +80,7 @@ KexiCSVExportWizard::KexiCSVExportWizard( const Options& options,
 		}
 	}
 	else if (m_tableOrQuery->query()) {
-		if (m_options.mode==Clipboard) {
+		if (m_options.mode==KexiCSVExport::Clipboard) {
 			setCaption(i18n("Copy Data From Query to Clipboard"));
 			infoLblFromText = i18n("Copying data from table:");
 		}
@@ -97,7 +90,7 @@ KexiCSVExportWizard::KexiCSVExportWizard( const Options& options,
 		}
 	}
 	else {
-		msgh.showErrorMessage(m_mainWin->project()->dbConnection(), 
+		msgh.showErrorMessage(m_mainWin->project()->dbConnection(),
 			i18n("Could not open data for exporting."));
 		m_cancelled = true;
 		return;
@@ -107,7 +100,7 @@ KexiCSVExportWizard::KexiCSVExportWizard( const Options& options,
 	// Setup pages
 
 	// 1. File Save Page
-	if (m_options.mode==File) {
+	if (m_options.mode==KexiCSVExport::File) {
 		m_fileSavePage = new KexiStartupFileDialog(
 			":CSVImportExport", //startDir
 			KexiStartupFileDialog::Custom | KexiStartupFileDialog::SavingFileBasedDB,
@@ -117,12 +110,12 @@ KexiCSVExportWizard::KexiCSVExportWizard( const Options& options,
 		m_fileSavePage->setDefaultExtension("csv");
 		m_fileSavePage->setLocationText( KexiUtils::stringToFileName(m_tableOrQuery->captionOrName()) );
 		connect(m_fileSavePage, SIGNAL(rejected()), this, SLOT(reject()));
-		addPage(m_fileSavePage, i18n("Enter the Name of the File You Want To Save the Data To"));
+		addPage(m_fileSavePage, i18n("Enter Name of File You Want to Save Data To"));
 	}
 
 	// 2. Export options
 	m_exportOptionsPage = new QWidget(this, "m_exportOptionsPage");
-	QGridLayout *exportOptionsLyr = new QGridLayout( m_exportOptionsPage, 6, 3, 
+	QGridLayout *exportOptionsLyr = new QGridLayout( m_exportOptionsPage, 6, 3,
 		KDialogBase::marginHint(), KDialogBase::spacingHint(), "exportOptionsLyr");
 	m_infoLblFrom = new KexiCSVInfoLabel( infoLblFromText, m_exportOptionsPage );
 	KexiPart::Info *partInfo = Kexi::partManager().infoForMimeType(
@@ -133,27 +126,27 @@ KexiCSVExportWizard::KexiCSVExportWizard( const Options& options,
 	exportOptionsLyr->addMultiCellWidget(m_infoLblFrom, 0, 0, 0, 2);
 
 	m_infoLblTo = new KexiCSVInfoLabel(
-		(m_options.mode==File) ? i18n("To CSV file:") : i18n("To clipboard:"),
+		(m_options.mode==KexiCSVExport::File) ? i18n("To CSV file:") : i18n("To clipboard:"),
 		m_exportOptionsPage
 	);
-	if (m_options.mode==Clipboard)
+	if (m_options.mode==KexiCSVExport::Clipboard)
 		m_infoLblTo->setIcon("editpaste");
 	exportOptionsLyr->addMultiCellWidget(m_infoLblTo, 1, 1, 0, 2);
 
-	m_showOptionsButton = new KPushButton(KGuiItem(i18n("Show Options >>"), "configure"), 
+	m_showOptionsButton = new KPushButton(KGuiItem(i18n("Show Options >>"), "configure"),
 		m_exportOptionsPage);
 	connect(m_showOptionsButton, SIGNAL(clicked()), this, SLOT(slotShowOptionsButtonClicked()));
 	exportOptionsLyr->addMultiCellWidget(m_showOptionsButton, 2, 2, 0, 0);
 	m_showOptionsButton->setSizePolicy(QSizePolicy::Fixed, QSizePolicy::Fixed);
 
 	// -<options section>
-	m_exportOptionsSection = new QGroupBox(1, Vertical, i18n("Options"), m_exportOptionsPage, 
+	m_exportOptionsSection = new QGroupBox(1, Vertical, i18n("Options"), m_exportOptionsPage,
 		"m_exportOptionsSection");
 	m_exportOptionsSection->setSizePolicy(QSizePolicy::Fixed, QSizePolicy::Fixed);
 	exportOptionsLyr->addMultiCellWidget(m_exportOptionsSection, 3, 3, 0, 1);
-	QWidget *exportOptionsSectionWidget 
+	QWidget *exportOptionsSectionWidget
 		= new QWidget(m_exportOptionsSection, "exportOptionsSectionWidget");
-	QGridLayout *exportOptionsSectionLyr = new QGridLayout( exportOptionsSectionWidget, 5, 2, 
+	QGridLayout *exportOptionsSectionLyr = new QGridLayout( exportOptionsSectionWidget, 5, 2,
 		0, KDialogBase::spacingHint(), "exportOptionsLyr");
 
 	// -delimiter
@@ -179,17 +172,17 @@ KexiCSVExportWizard::KexiCSVExportWizard( const Options& options,
 	m_characterEncodingCombo = new KexiCharacterEncodingComboBox( exportOptionsSectionWidget );
 	m_characterEncodingCombo->setSizePolicy(QSizePolicy::Fixed, QSizePolicy::Fixed);
 	exportOptionsSectionLyr->addWidget( m_characterEncodingCombo, 2, 1 );
-	QLabel *characterEncodingLabel = new QLabel(m_characterEncodingCombo, i18n("Text encoding:"), 
+	QLabel *characterEncodingLabel = new QLabel(m_characterEncodingCombo, i18n("Text encoding:"),
 		exportOptionsSectionWidget);
 	exportOptionsSectionLyr->addWidget( characterEncodingLabel, 2, 0 );
 
 	// - checkboxes
-	m_addColumnNamesCheckBox = new QCheckBox(i18n("Add column names as the first row"), 
+	m_addColumnNamesCheckBox = new QCheckBox(i18n("Add column names as the first row"),
 		exportOptionsSectionWidget);
 	m_addColumnNamesCheckBox->setChecked(true);
 	exportOptionsSectionLyr->addWidget( m_addColumnNamesCheckBox, 3, 1 );
 //! @todo 1.1: for copying use "Always use above options for copying" string
-	m_alwaysUseCheckBox = new QCheckBox(i18n("Always use above options for exporting"), 
+	m_alwaysUseCheckBox = new QCheckBox(i18n("Always use above options for exporting"),
 		m_exportOptionsPage);
 	exportOptionsLyr->addMultiCellWidget(m_alwaysUseCheckBox, 4, 4, 0, 1);
 //	exportOptionsSectionLyr->addWidget( m_alwaysUseCheckBox, 4, 1 );
@@ -198,16 +191,16 @@ KexiCSVExportWizard::KexiCSVExportWizard( const Options& options,
 	// -</options section>
 
 //	exportOptionsLyr->setColStretch(3, 1);
-	exportOptionsLyr->addMultiCell( 
+	exportOptionsLyr->addMultiCell(
 		new QSpacerItem( 0, 0, QSizePolicy::Preferred, QSizePolicy::MinimumExpanding), 5, 5, 0, 1 );
 
 //	addPage(m_exportOptionsPage, i18n("Set Export Options"));
-	addPage(m_exportOptionsPage, m_options.mode==Clipboard ? i18n("Copying") : i18n("Exporting"));
+	addPage(m_exportOptionsPage, m_options.mode==KexiCSVExport::Clipboard ? i18n("Copying") : i18n("Exporting"));
 	setFinishEnabled(m_exportOptionsPage, true);
 
 	// load settings
 	kapp->config()->setGroup("ImportExport");
-	if (m_options.mode!=Clipboard && readBoolEntry("ShowOptionsInCSVExportDialog", false)) {
+	if (m_options.mode!=KexiCSVExport::Clipboard && readBoolEntry("ShowOptionsInCSVExportDialog", false)) {
 		show();
 		slotShowOptionsButtonClicked();
 	}
@@ -222,14 +215,14 @@ KexiCSVExportWizard::KexiCSVExportWizard( const Options& options,
 		s = readEntry("DefaultEncodingForExportingCSVFiles");
 		if (!s.isEmpty())
 			m_characterEncodingCombo->setSelectedEncoding(s);
-		m_addColumnNamesCheckBox->setChecked( 
+		m_addColumnNamesCheckBox->setChecked(
 			readBoolEntry("AddColumnNamesForExportingCSVFiles", true) );
 	}
 
 	updateGeometry();
 
 	// -keep widths equal on page #2:
-	int width = QMAX( m_infoLblFrom->leftLabel()->sizeHint().width(), 
+	int width = QMAX( m_infoLblFrom->leftLabel()->sizeHint().width(),
 		m_infoLblTo->leftLabel()->sizeHint().width());
 	m_infoLblFrom->leftLabel()->setFixedWidth(width);
 	m_infoLblTo->leftLabel()->setFixedWidth(width);
@@ -245,13 +238,13 @@ bool KexiCSVExportWizard::cancelled() const
 	return m_cancelled;
 }
 
-void KexiCSVExportWizard::showPage ( QWidget * page ) 
+void KexiCSVExportWizard::showPage ( QWidget * page )
 {
 	if (page == m_fileSavePage) {
 		m_fileSavePage->setFocus();
 	}
 	else if (page==m_exportOptionsPage) {
-		if (m_options.mode==File)
+		if (m_options.mode==KexiCSVExport::File)
 			m_infoLblTo->setFileName( m_fileSavePage->currentFileName() );
 		QString text = m_tableOrQuery->captionOrName();
 		if (!m_rowCountDetermined) {
@@ -266,8 +259,8 @@ void KexiCSVExportWizard::showPage ( QWidget * page )
 		else
 			text += i18n("(columns: %1)").arg(columns);
 		m_infoLblFrom->setLabelText(text);
-		m_infoLblFrom->fileNameLabel()->setFixedHeight( 
-			QFontMetrics(m_infoLblFrom->fileNameLabel()->font()).height() * 2 );
+		QFontMetrics fm(m_infoLblFrom->fileNameLabel()->font());
+		m_infoLblFrom->fileNameLabel()->setFixedHeight( fm.height() * 2 + fm.lineSpacing() );
 		if (m_defaultsBtn)
 			m_defaultsBtn->show();
 	}
@@ -295,8 +288,12 @@ void KexiCSVExportWizard::next()
 void KexiCSVExportWizard::done(int result)
 {
 	if (QDialog::Accepted == result) {
-		
-		if (!exportData())
+		if (m_fileSavePage)
+			m_options.fileName = m_fileSavePage->currentFileName();
+		m_options.delimiter = m_delimiterWidget->delimiter();
+		m_options.textQuote = m_textQuote->textQuote();
+		m_options.addColumnNames = m_addColumnNamesCheckBox->isChecked();
+		if (!KexiCSVExport::exportData(*m_tableOrQuery, m_options))
 			return;
 	}
 	else if (QDialog::Rejected == result) {
@@ -305,7 +302,7 @@ void KexiCSVExportWizard::done(int result)
 
 	//store options
 	kapp->config()->setGroup("ImportExport");
-	if (m_options.mode!=Clipboard)
+	if (m_options.mode!=KexiCSVExport::Clipboard)
 		writeEntry("ShowOptionsInCSVExportDialog", m_exportOptionsSection->isVisible());
 	const bool store = m_alwaysUseCheckBox->isChecked();
 	writeEntry("StoreOptionsForCSVExportDialog", store);
@@ -377,160 +374,10 @@ void KexiCSVExportWizard::slotDefaultsButtonClicked()
 	m_characterEncodingCombo->selectDefaultEncoding();
 }
 
-bool KexiCSVExportWizard::exportData()
-{
-	KexiDB::Connection *conn = m_mainWin->project()->dbConnection();
-
-//! @todo move this to non-GUI location so it can be also used via command line
-//! @todo add a "finish" page with a progressbar.
-//! @todo look at m_rowCount whether the data is really large; 
-//!       if so: avoid copying to clipboard (or ask user) because of system memory
-
-//! @todo OPTIMIZATION: use fieldsExpanded(true /*UNIQUE*/)
-//! @todo OPTIMIZATION? (avoid multiple data retrieving) look for already fetched data within KexiProject..
-
-	KexiDB::QuerySchema* query = m_tableOrQuery->query();
-	if (!query)
-		query = m_tableOrQuery->table()->query();
-
-	KexiDB::QueryColumnInfo::Vector fields( query->fieldsExpanded() );
-	QString buffer;
-
-	KSaveFile *kSaveFile = 0;
-	QTextStream *stream = 0;
-
-	const bool copyToClipboard = m_options.mode==Clipboard;
-	if (copyToClipboard) {
-//! @todo (during exporting): enlarge bufSize by factor of 2 when it became too small
-		uint bufSize = QMIN((m_rowCount<0 ? 10 : m_rowCount) * fields.count() * 20, 128000);
-		buffer.reserve( bufSize );
-		if (buffer.capacity() < bufSize) {
-			kdWarning() << "KexiCSVExportWizard::exportData() cannot allocate memory for " << bufSize 
-				<< " characters" << endl;
-			return false;
-		}
-	}
-	else {
-		QString fname( m_fileSavePage->currentFileName() );
-		if (fname.isEmpty()) {//sanity
-			kdWarning() << "KexiCSVExportWizard::exportData(): fname is empty" << endl;
-			return false;
-		}
-		kSaveFile = new KSaveFile(m_fileSavePage->currentFileName());
-		if (0 == kSaveFile->status())
-			stream = kSaveFile->textStream();
-		if (0 != kSaveFile->status() || !stream) {//sanity
-			kdWarning() << "KexiCSVExportWizard::exportData(): status != 0 or stream == 0" << endl;
-			delete kSaveFile;
-			return false;
-		}
-	}
-
-//! @todo escape strings
-
-#define _ERR \
-	delete [] isText; \
-	if (kSaveFile) { kSaveFile->abort(); delete kSaveFile; } \
-	return false
-
-#define APPEND(what) \
-		if (copyToClipboard) buffer.append(what); else (*stream) << (what)
-
-// line endings should be as in RFC 4180
-#define CSV_EOLN "\r\n"
-
-	// 0. Cache information
-	const uint fieldsCount = fields.count();
-	const QCString delimiter( m_delimiterWidget->delimiter().left(1).latin1() );
-	const bool hasTextQuote = !m_textQuote->textQuote().isEmpty();
-	const QCString textQuote( m_textQuote->textQuote().left(1).latin1() );
-	const QCString escapedTextQuote( textQuote + textQuote );
-	//cache for faster checks
-	bool *isText = new bool[fieldsCount]; 
-	bool *isDateTime = new bool[fieldsCount]; 
-//	bool isInteger[fieldsCount]; //cache for faster checks
-//	bool isFloatingPoint[fieldsCount]; //cache for faster checks
-	for (uint i=0; i<fieldsCount; i++) {
-		isText[i] = fields[i]->field->isTextType();
-		isDateTime[i] = fields[i]->field->type()==KexiDB::Field::DateTime;
-//		isInteger[i] = fields[i]->field->isIntegerType() 
-//			|| fields[i]->field->type()==KexiDB::Field::Boolean;
-//		isFloatingPoint[i] = fields[i]->field->isFPNumericType();
-	}
-
-	// 1. Output column names
-	if (m_addColumnNamesCheckBox->isChecked()) {
-		for (uint i=0; i<fieldsCount; i++) {
-			if (i>0)
-				APPEND( delimiter );
-			if (hasTextQuote){
-				APPEND( textQuote );
-				APPEND( fields[i]->captionOrAliasOrName().replace(textQuote, escapedTextQuote) );
-				APPEND( textQuote );
-			}
-			else {
-				APPEND( fields[i]->captionOrAliasOrName() );
-			}
-		}
-		APPEND(CSV_EOLN);
-	}
-	
-	KexiGUIMessageHandler handler;
-	KexiDB::Cursor *cursor = conn->executeQuery(*query);
-	if (!cursor) {
-		handler.showErrorMessage(conn);
-		_ERR;
-	}
-	for (cursor->moveFirst(); !cursor->eof() && !cursor->error(); cursor->moveNext()) {
-		const uint realFieldCount = QMIN(cursor->fieldCount(), fieldsCount);
-		for (uint i=0; i<realFieldCount; i++) {
-			if (i>0)
-				APPEND( delimiter );
-			if (cursor->value(i).isNull())
-				continue;
-			if (hasTextQuote && isText[i]) {
-				APPEND( textQuote );
-				APPEND( QString(cursor->value(i).toString()).replace(textQuote, escapedTextQuote) );
-				APPEND( textQuote );
-			}
-			else if (isDateTime[i]) { //avoid "T" in ISO DateTime
-				APPEND( cursor->value(i).toDateTime().date().toString(Qt::ISODate)+" "
-					+ cursor->value(i).toDateTime().time().toString(Qt::ISODate) );
-			}
-			else {
-				APPEND( cursor->value(i).toString() );
-			}
-		}
-		APPEND(CSV_EOLN);
-	}
-
-	if (copyToClipboard)
-		buffer.squeeze();
-
-	if (!conn->deleteCursor(cursor)) {
-		handler.showErrorMessage(conn);
-		_ERR;
-	}
-
-	if (copyToClipboard)
-		kapp->clipboard()->setText(buffer, QClipboard::Clipboard);
-
-	delete [] isText;
-
-	if (kSaveFile) {
-		if (!kSaveFile->close()) {
-			kdWarning() << "KexiCSVExportWizard::exportData(): error close(); status == " 
-				<< kSaveFile->status() << endl;
-		}
-		delete kSaveFile;
-	}
-	return true;
-}
-
-static QString convertKey(const char *key, KexiCSVExportWizard::Mode mode)
+static QString convertKey(const char *key, KexiCSVExport::Mode mode)
 {
 	QString _key(QString::fromLatin1(key));
-	if (mode == KexiCSVExportWizard::Clipboard) {
+	if (mode == KexiCSVExport::Clipboard) {
 		_key.replace("Exporting", "Copying");
 		_key.replace("Export", "Copy");
 		_key.replace("CSVFiles", "CSVToClipboard");
@@ -565,7 +412,7 @@ void KexiCSVExportWizard::deleteEntry(const char *key)
 
 QString KexiCSVExportWizard::defaultDelimiter() const
 {
-	if (m_options.mode==Clipboard) {
+	if (m_options.mode==KexiCSVExport::Clipboard) {
 		if (!m_options.forceDelimiter.isEmpty())
 			return m_options.forceDelimiter;
 		else
@@ -576,9 +423,9 @@ QString KexiCSVExportWizard::defaultDelimiter() const
 
 QString KexiCSVExportWizard::defaultTextQuote() const
 {
-	if (m_options.mode==Clipboard)
-		return QString::null;
-	return KEXICSV_DEFAULT_TEXT_QUOTE;
+	if (m_options.mode==KexiCSVExport::Clipboard)
+		return KEXICSV_DEFAULT_CLIPBOARD_TEXT_QUOTE;
+	return KEXICSV_DEFAULT_FILE_TEXT_QUOTE;
 }
 
 #include "kexicsvexportwizard.moc"
