@@ -14,7 +14,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  */
 #ifndef ZOOM_PLUGIN_H
 #define ZOOM_PLUGIN_H
@@ -32,7 +32,7 @@ public:
   ZoomToolFactory( QObject* parent = 0, const char* name = 0 );
   ~ZoomToolFactory();
 
-  virtual QObject* create( QObject* parent = 0, const char* name = 0,
+  virtual QObject* createObject( QObject* parent = 0, const char* name = 0,
                            const char* classname = "QObject", const QStringList &args = QStringList() );
   static KInstance* global();
 
