@@ -14,8 +14,8 @@
 
    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
-   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-   Boston, MA 02111-1307, USA.
+   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
 */
 
 #ifndef KFORMULAMATHMLREAD_H
@@ -44,12 +44,18 @@ public:
 
     /**
      * Build a MathML 2 KFormula converter.
-     * call @startConversion to convert and wait for
-     * a @conversionFinished signal, than call
-     * @getKFormulaDom to get the converted DOM
+     * call startConversion() to convert and wait for
+     * a conversionFinished() signal, then call
+     * getKFormulaDom() to get the converted DOM
      */
     MathML2KFormula( const QDomDocument& mmldoc, const ContextStyle &contextStyle, bool oasisFormat = false );
 
+	/**
+	 * Build a MathML 2 KFormula converter.
+	 * A DOM Element is required instead of a full Document, useful for embedded MathML
+	 */ 
+    MathML2KFormula( const QDomElement& mmelm, const ContextStyle &contextStyle, bool oasisFormat = false );
+
     /*
      * Get the just created DOM.
      */
@@ -68,7 +74,7 @@ private:
     bool processElement( QDomNode node, QDomDocument& doc,
                          QDomNode docnode );
 
-    QDomDocument origdoc;
+    QDomElement orig_element;
     QDomDocument formuladoc;
     bool oasisFormat;
     const ContextStyle& context;
