@@ -14,8 +14,8 @@
 
    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
-   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-   Boston, MA 02111-1307, USA.
+   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
 */
 
 #ifndef SEQUENCEELEMENT_H
@@ -94,9 +94,10 @@ public:
      * Calculates our width and height and
      * our children's parentPosition.
      */
-    virtual void calcSizes(const ContextStyle& context,
-                           ContextStyle::TextStyle tstyle,
-                           ContextStyle::IndexStyle istyle);
+    virtual void calcSizes( const ContextStyle& style,
+						    ContextStyle::TextStyle tstyle,
+						    ContextStyle::IndexStyle istyle,
+							StyleAttributes& style );
 
     /**
      * Draws the whole element including its children.
@@ -107,6 +108,7 @@ public:
                        const ContextStyle& context,
                        ContextStyle::TextStyle tstyle,
                        ContextStyle::IndexStyle istyle,
+					   StyleAttributes& style,
                        const LuPixelPoint& parentOrigin );
 
     /**
@@ -115,7 +117,7 @@ public:
     virtual void dispatchFontCommand( FontCommand* cmd );
 
     virtual void drawEmptyRect( QPainter& painter, const ContextStyle& context,
-                                const LuPixelPoint& upperLeft );
+								double factor, const LuPixelPoint& upperLeft );
 
     virtual void calcCursorSize( const ContextStyle& context,
                                  FormulaCursor* cursor, bool smallCursor );
@@ -124,7 +126,8 @@ public:
      * If the cursor is inside a sequence it needs to be drawn.
      */
     virtual void drawCursor( QPainter& painter, const ContextStyle& context,
-                             FormulaCursor* cursor, bool smallCursor );
+							 StyleAttributes& style, FormulaCursor* cursor,
+							 bool smallCursor, bool activeCursor );
 
     // navigation
     //
@@ -192,6 +195,12 @@ public:
      */
     virtual void goInside(FormulaCursor* cursor);
 
+    /**
+     * Sets the cursor inside this element to its last position.
+     * For most elements that is the main child.
+     */
+    virtual void goInsideLast(FormulaCursor* cursor);
+
 
     // children
 
@@ -237,7 +246,7 @@ public:
     /**
      * @returns the number of children we have.
      */
-    int countChildren() const { return children.count(); }
+    uint countChildren() const { return children.count(); }
 
     /**
      * @returns whether the child has the given number.
@@ -283,6 +292,11 @@ public:
      */
     void getChildrenDom( QDomDocument& doc, QDomElement elem, uint from, uint to);
 
+    /**
+     * Stores the given childrens MathML dom in the element.
+     */
+    void getChildrenMathMLDom( QDomDocument& doc, QDomNode& elem, uint from, uint to );
+
     /**
      * Builds elements from the given node and its siblings and
      * puts them into the list.
@@ -298,13 +312,11 @@ public:
 
     virtual QString formulaString();
 
-    virtual void writeMathML( QDomDocument& doc, QDomNode parent, bool oasisFormat = false );
-
     /**
      * @returns the child at position i.
      */
     BasicElement* getChild(uint i) { return children.at(i); }
-    //const BasicElement* getChild(uint i) const { return children.at(i); }
+    const BasicElement* getChild(uint i) const;
 
     int childPos( BasicElement* child ) { return children.find( child ); }
     int childPos( const BasicElement* child ) const;
@@ -330,6 +342,9 @@ public:
         BasicElement& operator* ()
             { return *m_sequence->getChild( m_pos ); }
 
+        BasicElement* operator->() const
+            { return m_sequence->getChild( m_pos ); }
+
         ChildIterator& operator++ ()
             { ++m_pos; return *this; }
         ChildIterator operator++ ( int )
@@ -355,6 +370,11 @@ public:
 
     static void setCreationStrategy( ElementCreationStrategy* strategy );
 
+	virtual void setStyle(StyleElement *style);
+	virtual int buildChildrenFromMathMLDom(QPtrList<BasicElement>& list, QDomNode n);
+	virtual int readContentFromMathMLDom(QDomNode& node);
+	int buildMathMLChild( QDomNode node );
+
 protected:
 
     //Save/load support
@@ -369,6 +389,10 @@ protected:
      */
     virtual void writeDom(QDomElement element);
 
+    virtual QString getElementName() const { return "mrow"; }
+    virtual void writeMathMLContent( QDomDocument& doc, 
+                                     QDomElement& element,
+                                     bool oasisFormat ) const;
     /**
      * Reads our attributes from the element.
      * Returns false if it failed.
@@ -395,11 +419,12 @@ protected:
      *
      * @param type the desired type of the element
      */
-    virtual BasicElement* createElement(QString type);
+    virtual BasicElement* createElement(QString type, const QDomElement &element);
 
     /**
      * @returns the position where the child starts.
      *
+     * @param context the context the child is in
      * @param child the child's number
      */
     luPixel getChildPosition( const ContextStyle& context, uint child );
@@ -409,6 +434,27 @@ protected:
      */
     virtual bool isFirstOfToken( BasicElement* child );
 
+    /**
+     * Insert a new child in the sequence
+     * 
+     * @returns true if succesful, i.e. if index is in range, otherwise returns
+     * false. The valid range is 0 to count(). The child is appended if index == count().
+     *
+     * @param index position in the sequence to insert the child
+     * @param child the child to insert in the sequence
+     */
+    bool insert( uint index, BasicElement *child );
+
+    static ElementCreationStrategy* getCreationStrategy() { return creationStrategy; }
+
+    /**
+     * Space around sequence
+     */
+    virtual luPt getSpaceBefore( const ContextStyle&, ContextStyle::TextStyle, double ) { return 0; }
+    virtual luPt getSpaceAfter( const ContextStyle&, ContextStyle::TextStyle, double ) { return 0; }
+
+    static ElementCreationStrategy* creationStrategy;
+    
 private:
 
     /**
@@ -433,9 +479,9 @@ private:
      */
     bool textSequence;
 
-    static ElementCreationStrategy* creationStrategy;
-    
     bool singlePipe; //The key '|' produces one '|' not '| |', '||' produces '| |'
+
+	StyleElement *style;
 };
 
 
@@ -485,7 +531,8 @@ public:
      * If the cursor is inside a sequence it needs to be drawn.
      */
     virtual void drawCursor( QPainter& painter, const ContextStyle& context,
-                             FormulaCursor* cursor, bool smallCursor );
+							 StyleAttributes& style, FormulaCursor* cursor, 
+							 bool smallCursor, bool activeCursor );
 
     /**
      * Moves to the beginning of this word or if we are there already
@@ -533,8 +580,6 @@ public:
      */
     static bool isValidSelection( FormulaCursor* cursor );
 
-    virtual void writeMathML( QDomDocument& doc, QDomNode parent, bool oasisFormat = false );
-
 protected:
 
     /**
@@ -542,6 +587,7 @@ protected:
      */
     virtual QString getTagName() const { return "NAMESEQUENCE"; }
 
+    virtual QString getElementName() const { return "mi"; }
     /**
      * Creates a new element with the given type.
      *
